"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.migrateInTransaction = exports.PGStorage = void 0;
const pg_1 = require("pg");
const pg_escape_1 = require("pg-escape");
const configDefaults = {
    tableName: "schema_migration",
    columnName: "revision_id",
};
class PGStorage {
    constructor(db, partialConfig = {}) {
        this.tableCreated = false;
        this.db = db;
        this.config = { ...configDefaults, ...partialConfig };
    }
    async executed() {
        await this.createTable();
        const result = await this.db.query({
            text: `
        SELECT ${pg_escape_1.ident(this.config.columnName)}
        FROM ${pg_escape_1.ident(this.config.tableName)}
        ORDER BY ${pg_escape_1.ident(this.config.columnName)} ASC`,
            rowMode: "array",
        });
        return result.rows.flat();
    }
    async logMigration(migrationName) {
        await this.createTable();
        await this.db.query({
            text: `
        INSERT INTO ${pg_escape_1.ident(this.config.tableName)}
          (${pg_escape_1.ident(this.config.columnName)})
          VALUES ($1)
          ON CONFLICT DO NOTHING`,
            values: [migrationName],
        });
    }
    async unlogMigration(migrationName) {
        await this.createTable();
        await this.db.query({
            text: `
        DELETE FROM ${pg_escape_1.ident(this.config.tableName)}
        WHERE ${pg_escape_1.ident(this.config.columnName)} = $1`,
            values: [migrationName],
        });
    }
    async createTable() {
        if (this.tableCreated) {
            return;
        }
        await this.db.query(`
      CREATE TABLE IF NOT EXISTS ${pg_escape_1.ident(this.config.tableName)} (
        ${pg_escape_1.ident(this.config.columnName)} TEXT NOT NULL PRIMARY KEY UNIQUE
      )`);
        this.tableCreated = true;
    }
}
exports.PGStorage = PGStorage;
async function migrateInTransaction(db, cb) {
    let conn;
    if (db instanceof pg_1.Pool) {
        conn = await db.connect();
    }
    else {
        conn = db;
    }
    try {
        await conn.query("BEGIN TRANSACTION");
        const result = await cb(conn);
        await conn.query("COMMIT");
        return result;
    }
    catch (e) {
        await conn.query("ROLLBACK");
        throw e;
    }
    finally {
        if (db instanceof pg_1.Pool) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore release() does not exist on BaseClient
            conn.release();
        }
    }
}
exports.migrateInTransaction = migrateInTransaction;
//# sourceMappingURL=index.js.map